#!/usr/bin/env bash

if [ "$S_LIB_MODULE_LOADED" != "LOADED" ]; then  # avoid multiple loading of the same files
S_LIB_MODULE_LOADED="LOADED"
TRUE=0
FALSE=1

STAGES_INTERFACE_PATH=""


# this function takes two parameters:
# @param exitcode - it's a last operation exit code 0 -succees
# @param msg - this is optional message to be displayed on error
exit_on_error(){
    local exitcode=$1
    local msg=$2

    if [ $exitcode != 0  ]; then
        echo "Error" $msg
        exit $exitcode
    fi
}


# this is a library function which returns value by the echoing it
myreturn(){
    local  myresult=$1
    local  __resultvar=$2

    if [[ "$__resultvar" ]]; then
        eval $__resultvar="'$myresult'"
    else
        echo "$myresult"
    fi
}


function get_module(){
    local module=$1
    local git=$2
    local path=$3
    local tag=$4
    local branch=$5
    local include="$6"
    local prevdir=`pwd`
    local branchToCheckout=""

    

    if [ $module == "" ] || [ $git == "" ] || [ $path == "" ] ; then
        echo "Trying to load private package $module but you need to declare git and path variables"
    fi

    echo
    echo "---------------------$module----------------------------------"
    echo "Updating depenencies for private from $git"
    if [ "$tag" != "" ] ; then
        echo "It loads data from tag:$tag"
    fi
    if [ "$branch" != "" ] ; then
        echo "It loads data from branch:$branch"
    fi

    echo "It saves data into path $path"

    # first create missing direcotry
    if [ ! -d "$path" ] ; then
        mkdir -p "$path"
        exit_on_error $? "Can't create missing directory $path"
    fi

    if [ "$tag" != "" ]; then
        local branchToCheckout="$tag"
    fi
    if [ "$branch" != "" ]; then
        local branchToCheckout="$branch"
    fi


    # now checks if there is .git structre
    if [ ! -d "$path/.git" ]; then
        git clone --branch "$branchToCheckout" "$git" "$path"
        exit_on_error $? "Can't clone git $git into directory $path"
    else
        echo "There is alrady checkouted module"
        cd $path
        git fetch --all
        git checkout -f $branchToCheckout
        git pull
        cd $prevdir
    fi

    # this will eliminate files/folders that we don't put to "include" parameter
    if [ -d "$path" ]; then
        if [ "$include" != "" ]; then
            include=${include//,/ }
            findcommand="find $path/* -maxdepth 0"
            echo "$include"
            for includepart in $include
            do
                findcommand="$findcommand -not -name '$includepart'"
            done
            echo "Using find: $findcommand"
            folders=$(eval $findcommand)
            for folder in $folders
            do
                rm -rf $folder
            done
        fi
    fi

    echo "------------------------------------------------------------"
    echo
    cd "$prevdir"
}

function trim() {
    local var="$*"
    # remove leading whitespace characters
    var="${var#"${var%%[![:space:]]*}"}"
    # remove trailing whitespace characters
    var="${var%"${var##*[![:space:]]}"}"   
    printf '%s' "$var"
}


function is_stage_interfaces_in_the_expected_version(){
    local path="$1"
    local tag="$2"
    local rightVersion="false"

    if [ "$tag" == "" ]; then  # when no tag is specified we return false
        myreturn $rightVersion 
    else
        # we check if the module is in the taged version
        if [ -f "$path/package.json" ]; then 
            while read LINE
            do  
                LINE=$(trim "$LINE")
                if [ "${LINE:1:7}" == "version" ]; then
                    IFS=':' read -ra ADDR <<< "$LINE"
                    for i in "${ADDR[@]}"; do
                        i=$(trim "$i")
                        i="${i:1}"
                        i="${i:0:${#tag}}"
                        if [ "$i" == "$tag" ]; then
                            rightVersion="true"
                        fi
                    done
                fi
            done < "$path/package.json"
        else
            echo "I can't find $path/package.json"
        fi
    fi
    myreturn $rightVersion
}


function get_stages_interface_module(){
    local module=$1
    local git=$2
    local path=$3
    local tag=$4
    local branch=$5
    local include="$6"
    

    if [ "$module" == "stages-interface" ]; then
        local rightVersion
        rightVersion=$(is_stage_interfaces_in_the_expected_version "$path" "$tag")
        if [ "$rightVersion" != "true" ]; then
            get_module "$section" "$git" "$path" "$tag" "$branch" "$include"
        else
            echo "Stages interface is in the right version."
        fi
        STAGES_INTERFACE_PATH="$path"
    fi
}

function update_private_modules(){
    local inipath=$1
    # if the file don't exist just skip the execution of the function
    if [ ! -f "$inipath" ]; then
        return
    fi

    counter=0
    section="ś"
    tag=""
    branch=""
    path=""
    git=""
    include=""
    while IFS='= ' read var val
    do
        if [[ $var == \[*] ]]
        then
            if [ $counter -gt 0 ]; then # we are switching sections
                get_stages_interface_module "$section" "$git" "$path" "$tag" "$branch" "$include"
            fi
            section=${var:1}
            section=${section:0:$(expr ${#section} - 1)}
            let counter=counter+1
            tag=""
            branch=""
            path=""
            git=""
            include=""
        elif [[ $val ]]
        then
            if [ $var == "git" ]; then
                git=$val
            fi
            if [ $var == "tag" ]; then
                tag=$val
            fi
            if [ $var == "branch" ]; then
                branch=$val
            fi
            if [ $var == "path" ]; then
                path=$val
            fi
            if [ $var == "include" ]; then
                include="$val"
            fi
        fi
    done < $inipath
    if [ $counter -gt 0 ]; then # we are switching sections        
        get_stages_interface_module "$section" "$git" "$path" "$tag" "$branch" "$include"
    fi
}

function update_known_hosts() {
	# remove existing/old ssh keys
	ssh-keygen -R "[git.linguahouse.com]:7999"
	ssh-keygen -R git.linguahouse.com
	# add new hashed keys
	ssh-keyscan -H -p 7999 git.linguahouse.com >> ~/.ssh/known_hosts
	ssh-keyscan -H git.linguahouse.com >> ~/.ssh/known_hosts
}

update_private_modules "packages.ini"
update_known_hosts

if [ -f "$STAGES_INTERFACE_PATH/s-lib.sh" ]; then
    echo "Loading stages interface library..."
    source $STAGES_INTERFACE_PATH/s-lib.sh
else
    echo "Error. I can't load stages interface library!"
    exit 1
fi

fi #if [ ! -v $S_LIB_MODULE_LOADED ]; then
